@@ -1,4 +1,4 @@
-dd_SR_ML = function(brts,initparsopt=c(0.5,0.1,2*(1+length(brts)+missnumspec),2*(1+length(brts)+missnumspec),max(brts)/2),parsfix = NULL,idparsopt=c(1:3,6:7),idparsfix=NULL,idparsnoshift=(1:7)[c(-idparsopt,(-1)^(length(idparsfix) != 0) * idparsfix)],res=10*(1 + length(brts) + missnumspec),ddmodel=1,missnumspec=0,cond = TRUE,btorph = 1,allbp = FALSE)
+dd_SR_ML = function(brts, initparsopt = c(0.5,0.1,2*(1+length(brts)+missnumspec),2*(1+length(brts)+missnumspec),max(brts)/2), parsfix = NULL, idparsopt = c(1:3,6:7), idparsfix = NULL, idparsnoshift = (1:7)[c(-idparsopt,(-1)^(length(idparsfix) != 0) * idparsfix)], res = 10*(1 + length(brts) + missnumspec), ddmodel = 1, missnumspec = 0, cond = TRUE, btorph = 1, allbp = FALSE, tol = c(1E-3, 1E-4, 1E-6), maxiter = 1000 * round((1.25)^length(idparsopt)))
 {
 # brts = branching times (positive, from present to past)
 # - max(brts) = crown age
@@ -25,13 +25,26 @@ dd_SR_ML = function(brts,initparsopt=c(0.5,0.1,2*(1+length(brts)+missnumspec),2*
 # - cond = conditioning on non-extinction of the phylogeny
 # - btorph = likelihood of branching times (0) or phylogeny (1), differ by a factor (S - 1)! where S is the number of extant species
 # - allbp = optimize likelihood for fixed tshift at all bp (TRUE) or by letting tshift vary freely (FALSE)
+# - tol = tolerance in optimization
+#  . reltolx = relative tolerance of parameter values in optimization
+#  . reltolf = relative tolerance of function value in optimization
+#  . abstolx = absolute tolerance of parameter values in optimization
+# - maxiter = the maximum number of iterations in the optimization
 
 brts = sort(abs(as.numeric(brts)),decreasing = TRUE)
 options(warn=-1)
-if(is.numeric(brts) == FALSE) { cat("The branching times should be numeric") } else {
+if(is.numeric(brts) == FALSE)
+{
+   cat("The branching times should be numeric")
+   out2 = data.frame(row.names = "results",lambda_1 = -1, mu_1 = -1, K_1 = -1, lambda_2 = -1, mu_2 = -1, K_2 = -1, t_shift = -1, loglik = -1, df = -1, conv = -1)
+} else {
 idparsnoshift = sort(idparsnoshift)
 idpars = sort(c(idparsopt,idparsfix,idparsnoshift))
-if(sum(idpars == (1:7)) != 7) {cat("The parameters to be optimized, fixed and not shifted are incoherent.") } else {
+if(sum(idpars == (1:7)) != 7)
+{
+   cat("The parameters to be optimized, fixed and not shifted are incoherent.")
+   out2 = data.frame(row.names = "results",lambda_1 = -1, mu_1 = -1, K_1 = -1, lambda_2 = -1, mu_2 = -1, K_2 = -1, t_shift = -1, loglik = -1, df = -1, conv = -1)
+} else {
 namepars = c("la","mu","K","la2","mu2","K2","tshift")
 if(length(namepars[idparsopt]) == 0) { optstr = "nothing" } else { optstr = namepars[idparsopt] }
 cat("You are optimizing",optstr,"\n")
@@ -41,14 +54,30 @@ if(length(namepars[idparsnoshift]) == 0) { noshiftstr = "anything" } else { nosh
 cat("You are not shifting",noshiftstr,"\n")
 trparsopt = initparsopt/(1 + initparsopt)
 trparsfix = parsfix/(1 + parsfix)
+pars2 = c(res,ddmodel,cond,btorph,0,tol,maxiter)
+flush.console()
+initloglik = dd_SR_loglik_choosepar(trparsopt = trparsopt,trparsfix = trparsfix,idparsopt = idparsopt,idparsfix = idparsfix,idparsnoshift = idparsnoshift,pars2 = pars2,brts = brts,missnumspec = missnumspec)
+cat("The likelihood for the initial parameter values is",initloglik,"\n")
+if(initloglik == -Inf)
+{
+   cat("The initial parameter values have too low likelihood. Try again with different initial values.")
+   out2 = data.frame(row.names = "results",lambda_1 = -1, mu_1 = -1, K_1 = -1, lambda_2 = -1, mu_2 = -1, K_2 = -1, t_shift = -1, loglik = -1, df = -1, conv = -1)
+} else {
 cat("Optimizing the likelihood - this may take a while.","\n")
-out = optimx2(trparsopt,dd_SR_loglik_choosepar,hess=NULL,method = "Nelder-Mead",hessian = FALSE,control = list(maximize = TRUE,abstol = 1E-6,reltol = 1E-6,trace = 0,starttests = FALSE,kkt = FALSE),trparsfix = trparsfix,idparsopt = idparsopt,idparsfix = idparsfix,idparsnoshift = idparsnoshift,brts = brts,pars2 = c(res,ddmodel,cond,btorph),missnumspec = missnumspec)
-MLtrpars = unlist(out$par)
+flush.console()
+#code up to DDD v1.6: out = optimx2(trparsopt,dd_SR_loglik_choosepar,hess=NULL,method = "Nelder-Mead",hessian = FALSE,control = list(maximize = TRUE,abstol = pars2[8],reltol = pars2[7],trace = 0,starttests = FALSE,kkt = FALSE),trparsfix = trparsfix,idparsopt = idparsopt,idparsfix = idparsfix,idparsnoshift = idparsnoshift,brts = brts,pars2 = pars2,missnumspec = missnumspec)
+out = dd_SR_simplex(trparsopt,idparsopt,trparsfix,idparsfix,idparsnoshift,pars2,brts,missnumspec)
+if(out$conv > 0)
+{
+   cat("Optimization has not converged. Try again with different initial values.\n")
+   out2 = data.frame(row.names = "results",lambda_1 = -1, mu_1 = -1, K_1 = -1, lambda_2 = -1, mu_2 = -1, K_2 = -1, t_shift = -1, loglik = -1, df = -1, conv = unlist(out$conv))
+} else {
+MLtrpars = as.numeric(unlist(out$par))
 MLpars = MLtrpars/(1-MLtrpars)
 out$par = list(MLpars)
 MLpars1 = rep(0,7)
 MLpars1[idparsopt] = MLpars
-ML = out$fvalues
+ML = as.numeric(unlist(out$fvalues))
 if(sum(idparsfix == 7) == 0 && allbp == TRUE)
 { 
    idparsopt1 = idparsopt[1:(length(idparsopt) - 1)]
@@ -61,28 +90,31 @@ if(sum(idparsfix == 7) == 0 && allbp == TRUE)
          parsfix1 = c(idparsfix,brts[bp] + ba * 1E-8)
          trparsopt1 = initparsopt1/(1 + initparsopt1)
          trparsfix1 = parsfix1/(1 + parsfix1)
-         out = optimx2(trparsopt1,dd_SR_loglik_choosepar,hess=NULL,method = "Nelder-Mead",hessian = FALSE,control = list(maximize = TRUE,abstol = 1E-4,reltol = 1E-6,trace = 0,starttests = FALSE,kkt = FALSE),trparsfix = trparsfix1,idparsopt = idparsopt1,idparsfix = idparsfix1,idparsnoshift = idparsnoshift,brts = brts,pars2 = c(res,ddmodel,cond,btorph),missnumspec = missnumspec)
+         #code up to DDD v1.6: out = optimx2(trparsopt1,dd_SR_loglik_choosepar,hess=NULL,method = "Nelder-Mead",hessian = FALSE,control = list(maximize = TRUE,abstol = 1E-10,reltol = pars2[2],trace = 0,starttests = FALSE,kkt = FALSE),trparsfix = trparsfix1,idparsopt = idparsopt1,idparsfix = idparsfix1,idparsnoshift = idparsnoshift,brts = brts,pars2 = pars2,missnumspec = missnumspec)
+         out = dd_SR_simplex(trparsopt1,idparsopt1,trparsfix1,idparsfix1,idparsnoshift,pars2,brts,missnumspec)
          if(as.numeric(out$fvalues) > ML)
          {
-            MLtrpars = unlist(out$par)
+            MLtrpars = as.numeric(unlist(out$par))
             MLpars = MLtrpars/(1-MLtrpars)
             out$par = list(MLpars)
-            ML = out$fvalues
+            ML = as.numeric(unlist(out$fvalues))
          }
       }
    }
 }
-if(MLpars1[3] > 10^7){MLpars1[3] = Inf}
-if(MLpars1[6] > 10^7){MLpars1[6] = Inf}
 if(length(idparsfix) != 0) {MLpars1[idparsfix] = parsfix }
 if(length(idparsnoshift) != 0) { MLpars1[idparsnoshift] = MLpars1[idparsnoshift - 3] }
+if(MLpars1[3] > 10^7){MLpars1[3] = Inf}
+if(MLpars1[6] > 10^7){MLpars1[6] = Inf}
 s1 = sprintf('Maximum likelihood parameter estimates: %f %f %f %f %f %f %f',MLpars1[1],MLpars1[2],MLpars1[3],MLpars1[4],MLpars1[5],MLpars1[6],MLpars1[7])
-s2 = sprintf('Maxmimum loglikelihood: %f',ML)
+s2 = sprintf('Maximum loglikelihood: %f',ML)
 cat("\n",s1,"\n",s2,"\n")
 out$par = list(MLpars1)
 out$fvalues = list(ML)
-out2 = data.frame(row.names = "results",lambda_1 = MLpars1[1],mu_1 = MLpars1[2],K_1 = MLpars1[3],lambda_2 = MLpars1[4],mu_2 = MLpars1[5],K_2 = MLpars1[6],t_shift = MLpars1[7],loglik = unlist(out$fvalues),df = length(initparsopt),conv = unlist(out$conv))
+out2 = data.frame(row.names = "results",lambda_1 = MLpars1[1],mu_1 = MLpars1[2],K_1 = MLpars1[3],lambda_2 = MLpars1[4],mu_2 = MLpars1[5],K_2 = MLpars1[6],t_shift = MLpars1[7],loglik = ML,df = length(initparsopt),conv = unlist(out$conv))
+}
+}
+}
 }
 invisible(out2)
 }
-}
\ No newline at end of file
